"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    var desc = Object.getOwnPropertyDescriptor(m, k);
    if (!desc || ("get" in desc ? !m.__esModule : desc.writable || desc.configurable)) {
      desc = { enumerable: true, get: function() { return m[k]; } };
    }
    Object.defineProperty(o, k2, desc);
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.groupChangesets = exports.filterChangesets = exports.searchChangesets = exports.listChangesets = exports.FormatMode = exports.OutputMode = exports.GroupMode = exports.SearchMode = exports.getUnityChangeset = exports.UnityChangeset = void 0;
const dntShim = __importStar(require("./_dnt.shims.js"));
const unityChangeset_js_1 = require("./unityChangeset.js");
exports.UnityChangeset = unityChangeset_js_1.UnityChangeset;
const REGEXP_HUB_LINKS = /unityhub:\/\/\d{4}\.\d+\.\d+(a|b|f)\d+\/\w{12}/g;
const REGEXP_UNITY_VERSION = /\d{4}\.\d+\.\d+(a|b|f)\d+/g;
const UNITY_CHANGESETS_DB_URL = "https://mob-sakai.github.io/unity-changeset/db";
const UNITY_RSS_URLS = [
    "https://unity.com/releases/editor/lts-releases.xml",
    "https://unity.com/releases/editor/tech-and-preview-releases.xml",
    "https://unity.com/releases/editor/beta-releases.xml",
    "https://unity.com/releases/editor/alpha-releases.xml",
];
/*
Unity release URLs for each lifecycle.
*/
const UNITY_RELEASE_URLS = {
    "p": "https://unity.com/releases/editor/whats-new/",
    "f": "https://unity.com/releases/editor/whats-new/",
    "a": "https://unity.com/releases/editor/alpha/",
    "b": "https://unity.com/releases/editor/beta/",
};
/*
 * Get an Unity changeset from specific Unity version.
 * @param version The Unity version.
 * @returns An Unity changeset.
 */
async function getUnityChangeset(version) {
    const db = await loadDb();
    const results = db.filter((c) => c.version === version);
    return results.length > 0
        ? results[0]
        : getUnityChangesetFromReleasePage(version);
}
exports.getUnityChangeset = getUnityChangeset;
async function getUnityChangesetFromReleasePage(version) {
    const match = version.match(/^(\d+\.\d+\.\d+)(a|b|f|p)\d+$/);
    const lifecycle = match?.[2];
    const releaseUrl = UNITY_RELEASE_URLS[lifecycle];
    const shortVersion = match?.[1];
    const releasePageUrl = releaseUrl +
        (lifecycle == "f" ? shortVersion : version);
    const response = await dntShim.fetch(releasePageUrl);
    const text = await response.text();
    const matchLink = text.match(REGEXP_HUB_LINKS);
    if (!matchLink) {
        throw new Error(`No changeset found at '${releasePageUrl}'`);
    }
    const changeset = exports.UnityChangeset.createFromHref(matchLink[0]);
    if (changeset.version !== version) {
        throw new Error(`No changeset found at '${releasePageUrl}'`);
    }
    return changeset;
}
/*
 * Search mode.
 *
 * All: All changesets.
 * Default: Only non pre-release changesets.
 * PreRelease: Only pre-release (alpha/beta) changesets.
 */
var SearchMode;
(function (SearchMode) {
    SearchMode[SearchMode["All"] = 0] = "All";
    SearchMode[SearchMode["Default"] = 2] = "Default";
    SearchMode[SearchMode["PreRelease"] = 3] = "PreRelease";
})(SearchMode = exports.SearchMode || (exports.SearchMode = {}));
/*
 * Group mode.
 *
 * All: All the changesets.
 * OldestPatch: Only the oldest patch changesets.
 * LatestPatch: Only the latest patch changesets.
 * LatestLifecycle: Only the latest lifecycle changesets.
 */
var GroupMode;
(function (GroupMode) {
    GroupMode["All"] = "all";
    GroupMode["OldestPatch"] = "oldest-patch";
    GroupMode["LatestPatch"] = "latest-patch";
    GroupMode["LatestLifecycle"] = "latest-lifecycle";
})(GroupMode = exports.GroupMode || (exports.GroupMode = {}));
/*
 * Output mode.
 *
 * Changeset: The changeset.
 * VersionOnly: Only the version.
 * MinorVersionOnly: Only the minor version.
 */
var OutputMode;
(function (OutputMode) {
    OutputMode["Changeset"] = "changeset";
    OutputMode["VersionOnly"] = "version";
    OutputMode["MinorVersionOnly"] = "minor-version";
})(OutputMode = exports.OutputMode || (exports.OutputMode = {}));
/*
 * Format mode.
 *
 * None: No format.
 * Json: JSON format.
 * PrettyJson: Pretty JSON format.
 */
var FormatMode;
(function (FormatMode) {
    FormatMode["None"] = "none";
    FormatMode["Json"] = "json";
    FormatMode["PrettyJson"] = "pretty-json";
})(FormatMode = exports.FormatMode || (exports.FormatMode = {}));
function listChangesets(searchMode, filterOptions, groupMode, outputMode, formatMode) {
    return searchChangesets(searchMode)
        .then((results) => filterChangesets(results, filterOptions))
        .then((results) => groupChangesets(results, groupMode))
        .then((results) => {
        switch (outputMode) {
            case OutputMode.Changeset:
                return results;
            case OutputMode.VersionOnly:
                return results.map((c) => c.version);
            case OutputMode.MinorVersionOnly:
                return results.map((c) => c.minor);
            default:
                throw Error(`The given output mode '${outputMode}' was not supported`);
        }
    })
        .then((results) => {
        switch (formatMode) {
            case FormatMode.None:
                return results.join("\n");
            case FormatMode.Json:
                return JSON.stringify(results);
            case FormatMode.PrettyJson:
                return JSON.stringify(results, null, 2);
            default:
                throw Error(`The given format mode '${formatMode}' was not supported`);
        }
    });
}
exports.listChangesets = listChangesets;
async function loadDb() {
    const response = await dntShim.fetch(UNITY_CHANGESETS_DB_URL);
    const text = await response.text();
    const lines = text.split("\n");
    return lines
        .map((line) => exports.UnityChangeset.createFromDb(line));
}
async function findVersions(url) {
    const response = await dntShim.fetch(url);
    const text = await response.text();
    const lines = text.split("\n");
    const versions = lines
        .map((l) => l.match(REGEXP_UNITY_VERSION))
        .filter((m) => m)
        .map((m) => m?.[0]);
    return Array.from(new Set(versions));
}
async function searchChangesets(searchMode) {
    const results = await loadDb();
    const versions = await Promise.all(UNITY_RSS_URLS
        .map((url) => findVersions(url)));
    const appendResults = await Promise.all(versions
        .flat()
        .filter((v) => !results.some((r) => r.version === v))
        .map((v) => getUnityChangesetFromReleasePage(v)));
    const allResults = results.concat(appendResults);
    switch (searchMode) {
        case SearchMode.All:
            return allResults;
        case SearchMode.Default:
            return allResults
                .filter((c) => !c.preRelease);
        case SearchMode.PreRelease:
            return allResults
                .filter((c) => c.preRelease);
        default:
            throw Error(`The given search mode '${searchMode}' was not supported`);
    }
}
exports.searchChangesets = searchChangesets;
function filterChangesets(changesets, options) {
    if (!changesets || changesets.length == 0)
        return [];
    // Min version number
    const min = options.min
        ? exports.UnityChangeset.toNumber(options.min, false)
        : Number.MIN_VALUE;
    // Max version number
    const max = options.max
        ? exports.UnityChangeset.toNumber(options.max, true)
        : Number.MAX_VALUE;
    // Grep pattern
    const regex = options.grep ? new RegExp(options.grep, "i") : null;
    // Lifecycle filter
    const lc = options.allLifecycles
        ? null
        : Object.values(groupBy(changesets, (r) => r.minor))
            .map((g) => g[0]);
    return changesets
        .filter((c) => min <= c.versionNumber &&
        c.versionNumber <= max &&
        (!options.lts || c.lts) &&
        (!regex || regex.test(c.version)) &&
        (!lc || lc.some((l) => l.minor == c.minor && l.lifecycle == c.lifecycle)));
}
exports.filterChangesets = filterChangesets;
function groupChangesets(changesets, groupMode) {
    if (!changesets || changesets.length == 0)
        return [];
    switch (groupMode) {
        case GroupMode.All:
            return changesets;
        case GroupMode.LatestLifecycle:
            return Object.values(groupBy(changesets, (r) => r.minor))
                .map((g) => g.filter((v) => v.lifecycle == g[0].lifecycle))
                .flat();
        case GroupMode.LatestPatch:
            return Object.values(groupBy(changesets, (r) => r.minor))
                .map((g) => g[0]);
        case GroupMode.OldestPatch:
            return Object.values(groupBy(changesets, (r) => r.minor))
                .map((g) => g.filter((v) => v.lifecycle == g[0].lifecycle))
                .map((g) => g[g.length - 1]);
        default:
            throw Error(`The given group mode '${groupMode}' was not supported`);
    }
}
exports.groupChangesets = groupChangesets;
const groupBy = (arr, key) => arr.reduce((groups, item) => {
    (groups[key(item)] ||= []).push(item);
    return groups;
}, {});
